@@ -24,6 +24,7 @@
 #include <json-glib/json-glib.h>
 
 #include "bz-addons-dialog.h"
+#include "bz-app-size-dialog.h"
 #include "bz-decorated-screenshot.h"
 #include "bz-dynamic-list-view.h"
 #include "bz-env.h"
@@ -36,6 +37,7 @@
 #include "bz-screenshot.h"
 #include "bz-section-view.h"
 #include "bz-share-dialog.h"
+#include "bz-spdx.h"
 #include "bz-state-info.h"
 #include "bz-stats-dialog.h"
 
@@ -206,15 +208,42 @@ format_recent_downloads (gpointer object,
                          int      value)
 {
   if (value > 0)
-    return g_strdup_printf (_ ("%'d downloads in the last month"), value);
+    return g_strdup_printf (_ ("%'d Monthly Downloads"), value);
   else
-    return g_strdup ("---");
+    return g_strdup (_ ("--- Downloads"));
 }
 
 static char *
 format_size (gpointer object, guint64 value)
 {
-  return g_format_size (value);
+  g_autofree char *size_str = g_format_size (value);
+  char            *space    = g_strrstr (size_str, "\xC2\xA0");
+
+  if (space != NULL)
+    {
+      *space = '\0';
+      return g_strdup_printf ("%s <span font_size='x-small'>%s</span>",
+                              size_str, space + 2);
+    }
+
+  return g_strdup (size_str);
+}
+
+static char *
+format_license (gpointer    object,
+                const char *license)
+{
+  g_autofree char *name = NULL;
+
+  if (license == NULL || *license == '\0')
+    return g_strdup (_ ("Unknown"));
+
+  name = bz_spdx_get_name (license);
+
+  if (name != NULL && *name != '\0')
+    return g_steal_pointer (&name);
+  else
+    return g_strdup (license);
 }
 
 static char *
@@ -256,6 +285,16 @@ format_as_link (gpointer    object,
     return g_strdup (_ ("No URL"));
 }
 
+static gboolean
+has_link (gpointer    object,
+          const char *license)
+{
+  if (license == NULL || *license == '\0')
+    return FALSE;
+
+  return bz_spdx_is_valid (license);
+}
+
 static char *
 pick_license_warning (gpointer object,
                       gboolean value)
@@ -278,7 +317,28 @@ open_url_cb (BzFullView   *self,
   if (url != NULL && *url != '\0')
     g_app_info_launch_default_for_uri (url, NULL, NULL);
   else
-    g_warning ("Invalid or empty URL provided");
+    g_warning ("Invalid or empty URL provided for Flathub URL CB");
+}
+
+static void
+open_flathub_url_cb (BzFullView *self,
+                     GtkButton  *button)
+{
+  BzEntry    *entry = NULL;
+  const char *id    = NULL;
+  char       *url   = NULL;
+
+  entry = BZ_ENTRY (bz_result_get_object (self->ui_entry));
+  id    = bz_entry_get_id (entry);
+
+  if (id != NULL && *id != '\0')
+    {
+      url = g_strdup_printf ("https://flathub.org/apps/%s", id);
+      g_app_info_launch_default_for_uri (url, NULL, NULL);
+      g_free (url);
+    }
+  else
+    g_warning ("Invalid or empty ID provided");
 }
 
 static void
@@ -296,6 +356,31 @@ share_cb (BzFullView *self,
   adw_dialog_present (share_dialog, GTK_WIDGET (self));
 }
 
+static void
+license_cb (BzFullView *self,
+            GtkButton  *button)
+{
+  BzEntry         *entry   = NULL;
+  const char      *license = NULL;
+  g_autofree char *url     = NULL;
+
+  entry = bz_result_get_object (self->ui_entry);
+  if (entry == NULL)
+    return;
+
+  g_object_get (entry, "project-license", &license, NULL);
+
+  if (license == NULL || *license == '\0')
+    return;
+
+  url = bz_spdx_get_url (license);
+
+  if (url != NULL)
+    g_app_info_launch_default_for_uri (url, NULL, NULL);
+  else
+    g_warning ("Could not generate URL for license: %s", license);
+}
+
 static void
 dl_stats_cb (BzFullView *self,
              GtkButton  *button)
@@ -319,6 +404,19 @@ dl_stats_cb (BzFullView *self,
   bz_stats_dialog_animate_open (BZ_STATS_DIALOG (dialog));
 }
 
+static void
+size_cb (BzFullView *self,
+         GtkButton  *button)
+{
+  AdwDialog *size_dialog = NULL;
+
+  if (self->group == NULL)
+    return;
+
+  size_dialog = bz_app_size_dialog_new (bz_result_get_object (self->ui_entry));
+  adw_dialog_present (size_dialog, GTK_WIDGET (self));
+}
+
 static void
 run_cb (BzFullView *self,
         GtkButton  *button)
@@ -720,9 +818,14 @@ bz_full_view_class_init (BzFullViewClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, format_size);
   gtk_widget_class_bind_template_callback (widget_class, format_timestamp);
   gtk_widget_class_bind_template_callback (widget_class, format_as_link);
+  gtk_widget_class_bind_template_callback (widget_class, format_license);
+  gtk_widget_class_bind_template_callback (widget_class, has_link);
   gtk_widget_class_bind_template_callback (widget_class, open_url_cb);
+  gtk_widget_class_bind_template_callback (widget_class, open_flathub_url_cb);
+  gtk_widget_class_bind_template_callback (widget_class, license_cb);
   gtk_widget_class_bind_template_callback (widget_class, share_cb);
   gtk_widget_class_bind_template_callback (widget_class, dl_stats_cb);
+  gtk_widget_class_bind_template_callback (widget_class, size_cb);
   gtk_widget_class_bind_template_callback (widget_class, run_cb);
   gtk_widget_class_bind_template_callback (widget_class, install_cb);
   gtk_widget_class_bind_template_callback (widget_class, remove_cb);
